;-----------------------------------------------------
; zcode 7sgement display and switch interface
;
; z690a_7seg.inc 01/10/2007
;-----------------------------------------------------

;_frontPanelTest	clrf	column
;	clrf 	Data0
;	clrf	Data1
;_frontPanel	nop
;	movfw	Data0
;	call	_bin2bcd
;	call	_bcdTo7Seg	
;
;
;	btfss	PIR1, CCP1IF
;	goto	_frontPanel
;	bcf	PIR1,CCP1IF
;	call	_7segDriver
;	decf	Data1,F
;	skpnz
;	incf	Data0,F
;
;	goto	_frontPanel





;----------------------------------------------------------------------------------------------
; 7segment display multiplex driver
;
_7segDriver	incf	column,F	; shift through each digit column
	movfw	column
	addlw	-.3
	skpnc
	clrf	column
	
	movfw	FSR
	movwf	saveFSR
	movfw	column	; convert digit value to 7seg data
	addlw	digit1
	movwf	FSR
	call	_7segLookup
	btfsc	INDF,7	; test bit 7 of digit (if set we need decimal point)
	iorlw	0x80	; set bit 7 of 7seg data (which connects to DP)

	btfsc	flags2, LEDtype	; test for 7seg type [0 = CC, 1 = CA ]
	xorlw	0xFF
	movwf	shifter	; put in shifter variable

		
	bsf	outEN	; Hi-Z segment drive outputs
	movlw	.8
	movwf	shiftCounter



_shiftLoop	bcf	srDat	; set serial data == 0
	bcf	srClk	; and take low again
	rlf	shifter,F	; rotate shifter  left 1 bit
	skpnc		; skip if bit was 0 (no carry)
	bsf	srDat	; else set serial data == 1
	bsf	srClk	; rising edge will clock data in
	decfsz	shiftCounter,F	; count 8 bits shifted
	goto	_shiftLoop	; loop until complete

	
	bcf	rClk	; take rClk low
	bsf	rClk	; take it high to clock srLatch


	clrc
	rlf	column,w	; activate digit drive
	
	btfsc	flags2, LEDtype	
	call	_7segDriveCA
	btfss	flags2, LEDtype	
	call	_7segDriveCC
	
	bcf	outEN	; take output enable low
	
	clrf	PCLATH	; ensure PCLATH is clear as other expect it to be so
	movfw	saveFSR	; restore saved FSR value which is used in the serialRX code
	movwf	FSR
	return



_7segDriveCA	bsf	drDig3
	bsf	drDig2
	bsf	drDig1
	addwf	PCL,F
	bcf	drDig3
	return
	bcf	drDig2
	return
	bcf	drDig1
	return

_7segDriveCC	bcf	drDig3
	bcf	drDig2
	bcf	drDig1
	addwf	PCL,F
	bsf	drDig3
	return
	bsf	drDig2
	return
	bsf	drDig1
	return


	
	IF (HIGH _7segLookup != HIGH _7segDriveCA)
	error "7segDrive in zcode_7seg.inc crosses page boundary"
	ENDIF


_7segLookup	movlw	HIGH _7segLookup
	movwf	PCLATH
	movfw	INDF	; read value for current displayed digit
	andlw	0x7F	; mask out bit 7 which indicates decimal point
	addwf	PCL,F
	retlw	b'00111111'	; 0
	retlw	b'00000110'	; 1
	retlw	b'01011011'	; 2
	retlw	b'01001111'	; 3
	retlw	b'01100110'	; 4
	retlw	b'01101101'	; 5
	retlw	b'01111101'	; 6
	retlw	b'00000111'	; 7
	retlw	b'01111111'	; 8
	retlw	b'01101111'	; 9
	retlw	b'01110111'	; A
	retlw	b'01111100'	; b
	retlw	b'01011000'	; c
	retlw	b'01011110'	; d
	retlw	b'01111001'	; E
	retlw	b'01110001'	; F
	retlw	b'00000000'	; blank
	retlw	b'01010100'	; n
	retlw	b'01011100'	; o
	retlw	b'01101110'	; y

	
	IF (HIGH $ != HIGH _7segLookup)
	error "7 seg lookup table in zcode_7seg.inc crosses page boundary"
	ENDIF



;-------------------------------------------
; 8-bit binary to BCD conversion
; Enter with binary value for conversion in W 
; Packed BCD result returned in bcdLo, bcdHi

_bin2bcd	movwf	bin
	movlw	d'5'
  	movwf	tvar1
                	clrf	bcdLo
                	clrf	bcdHi 

           	rlf	bin,F
            	rlf	bcdLo,F
            	rlf	bin,F
            	rlf	bcdLo,F
                	rlf	bin,F
            	rlf	bcdLo,F

_repeat     	movfw	bcdLo
            	addlw	0x33
            	movwf	tvar2
            	movfw	bcdLo
            	btfsc	tvar2,3
            	addlw	0x03
            	btfsc	tvar2,7
            	addlw	0x30
            	movwf	bcdLo
            	rlf	bin,F
            	rlf	bcdLo,F
            	rlf	bcdHi,F
            	decfsz	tvar1,F
            	goto	 _repeat
	
            	return
;-------------------------------------------
; transfer BCD to Digits

_bcdTo7Seg	movfw	bcdHi
	movwf	digit3
	swapf	bcdLo,W
	andlw	0x0F
	movwf	digit2
	movfw	bcdLo
	andlw	0x0F
	movwf	digit1
	return