;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; zcode_reset.inc
; z690a 
; Initialisation for 16F690
; 09/10/2007
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

_main	clrf	STATUS
	

	bsf	STATUS,RP1
	clrf	ANSEL	; Set I/O ports for digital mode
	clrf	ANSELH
	bcf	STATUS,RP1
	bank1
	movlw	b'01110000'	; set OSCCON for internal RC oscillator
	movwf	OSCCON	; running at 8Mhz
	
	movlw	b'00000000'	; enable weak pull up on PORTA+B
	movwf	OPTION_REG

	movlw	b'11111111'
	movwf	TRISA
	movlw	b'11110111'
	movwf	WPUA

	movlw	.80	; initialise Timer 2 Period Register
	movwf	PR2	; for 50uS period 
			; 50uS x 256 = PWM refresh rate of 78Hz	
	
	movlw	b'00000100'	; enable High Speed Bit Rate
	movwf	TXSTA	; and leave serial TX disabled
	
	bsf	PIE1, TMR2IE	; enable Timer 2 interrupts



	movlw	b'01110000'
	movwf	TRISB
	bsf	STATUS, RP1	
	bcf	STATUS, RP0	; set to bank 2
	movwf	WPUB
	bcf	STATUS, RP1
	bcf	STATUS, RP0 	; set to bank 0
	
	movlw	b'10000000'	; Enable serial port
	movwf	RCSTA
	
	movlw 	b'00000100'	; Enable Timer 2 and configure for
	movwf	T2CON	; 1:1 pre and postscale

	; Startup PWM values
	movlw 	.0
	movwf	red
	movlw	.0
	movwf	grn
	movlw	.0
	movwf	blu	
	

	btfss	sw2	; test if sw2 held down at reset
	goto	_EEPROMbadsum	; initialise EEPROM if input sw2 down


	call	_EEPROMboot	; read settings from EEPROM

	movfw	eepromAddress	; load saved device address
	movwf	myAddress	; initialise device address variable
	andlw	0x0F	; calculate device Group Column Address
	iorlw	0x90
	movwf	myGroupCol	; and initialise group address variable
	
	swapf	myAddress,W	; calculate device Group Row Address
	andlw	0x0F
	iorlw	0x80
	movwf	myGroupRow

	movfw	eepromInactive
	movwf	activeTmrEn	; initialise inactivity timeout flag
	
	bank1
	movfw	eepromBitRate	; read saved bit rate
	movwf	savedBitRate	; keep copy of saved bit rate
	call	_bitRateLookup
	movwf	SPBRG	; and initialise SPBRG register
	xorlw	.103	; test if SPBRG value is .103 (1.2Kbps)
	skpnz		;
	bcf	TXSTA, BRGH	; Requires BRGH=0 for low speed
	clrf	BAUDCTL
	bank0
		
	movlw	b'00001011'	; configure Timer one for Compare mode
	movwf	CCP1CON	; and Special Even Trigger function
	movlw	LOW .7812	; Setup the compare registers to
	movwf	CCPR1L	; set the CCP1IF every 4mS
	movlw	HIGH .7812
	movwf	CCPR1H
	movlw	b'00000001'
	movwf	T1CON
	clrf	fadeflag
	clrf	flags	; initialise flag variable
	clrf	flags2	; initialise flags2 variable
	clrf	Chksum	; initialise checksum


	btfsc	activeDriveLevel; test PWM drive level input
	bsf	flags2, driveLevel  ; set bit if input high

	btfsc	segDriveLevel
	bsf	flags2, LEDtype	; set 7 seg drive for CA or CC [0 = CC, 1 =CA ]	

	movlw	rxBufferBase	; initialise rx buffer index pointer
	movwf	FSR
	clrf	PCLATH	; set PCLATH to page 0
	bank1
	movlw	b'00000000'
	movwf	TRISC
	bank0
	bsf	outEN	
	bcf	rClk

	bsf	INTCON, PEIE	; enable Peripheral interrupt
	bsf	INTCON, GIE	; enable Global interrupt

	movlw	.255
_delay	btfss	PIR1,CCP1IF	; 5mS timer tick
	goto	_delay
	bcf	PIR1, CCP1IF
	addlw	-.1
	skpz
	goto	_delay
	clrf	red
	clrf	grn
	clrf	blu
	clrf	column	; 7seg digit drive pointer
	bsf	RCSTA, CREN	; enable serial port for continuos receive

	;goto	_frontPanelTest
	clrf	fpState
	clrf	sleepTimerHi	
	clrf	sleepTimerLo

;---------------------------------------------------------------------------
; zcode_core
;
; core code loop
;
_coreLoop	btfsc	PIR1, RCIF	; is data ready in RX buffer
	call	_serialRX
	
	btfsc	PIR1,CCP1IF	; 5mS timer tick
	call	_timerTick
	
	btfsc	PIR1, RCIF	; is data ready in RX buffer
	call	_serialRX

	btfsc	flags, rxNewCmd	; has new command been received
	call	_command

	goto	_coreLoop
	
_timerTick	bcf	PIR1,CCP1IF	; clear timer 1 int' flag
	
	; Test for sw1 held for over 1 second 
	btfsc	sw1	; Skip next if sw1 down
	clrf	sw1timer	; clear sw1Timer if sw1 up
	btfss	sw1	; Skip next if sw1 up
	incfsz	sw1timer,F	; Increment sw1Timer if sw1 up, skip next if rolled-over
	goto	_fader	; goto _fader


_fpControl 	call	_dispAddr	; intialise 7seg display with current address
	clrf	fpState	; clear fpState (address =0,  bit rate =1, save mode =2)
	clrf	sw1timer	
	clrf	sw2timer
	bsf	flags2, sw1Last	; preset sw1Last flag
	
	btfss	PIR1, CCP1IF	; wait for sw1 to be released
	goto	$-1	
	bcf	PIR1, CCP1IF
	call	_7segDriver	; and multiplex 7 seg display
	btfss	sw1
	goto	$-5	
	

_fpLoop	btfss	PIR1, CCP1IF	
	goto	$-1
	bcf	PIR1, CCP1IF
	call	_7segDriver	; multiplex 7 seg display
	call	_testSw2	; test switch 2
	call	_testSw1	; test switch 1
	btfss	flags2, exitFP	; do we need to exit front panel setup?
	goto	_fpLoop	; no, then loop 

	bcf	flags2, exitFP	; yes, then clear flag
	; Hi Z 7seg segment drive
	bsf	outEN

	; wait for sw1 to be released 
	btfss	PIR1, CCP1IF	
	goto	$-1
	bcf	PIR1, CCP1IF	
	btfss	sw1
	goto	$-5

		
	; fade / hold state function select
	
_fader	movf	fadeflag,F	; test fade flag
	skpz		; skip next if zero
	call	_fading	; else call RGB fade function



_sleepTimer	incf	sleepTimerLo,F	; increment 16 bit sleep timer
	skpnz	
	incfsz	sleepTimerHi,F	
	return
	
	bcf	INTCON, GIE	; if sleep timer rolled over
	bcf	INTCON, GIE	; stop PWM interrupt
	movfw	LEDport
	iorlw	(1<<redLED | 1<<grnLED | 1<<bluLED)
	btfsc	activeDriveLevel
	andlw	~(1<<redLED | 1<<grnLED | 1<<bluLED)
	movwf	LEDport
	return


	



